# Generated by Django 4.2.24 on 2025-09-12 08:08

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("pretix_cashfree", "0001_initial"),
    ]

    operations = [
        migrations.DeleteModel(
            name="PaymentWebhookEvent",
        ),
    ]
