X_API_VERSION = "2025-01-01"
SANDBOX_CLIENT_KEY = "TEST430329ae80e0f32e41a393d78b923034"
SANDBOX_CLIENT_SECRET = "TESTaf195616268bd6202eeb3bf8dc458956e7192a85"
SESSION_KEY_ORDER_ID = "payment_cashfree_pid"
RETURN_URL_PARAM = "order_id"
REDIRECT_URL_PAYMENT_SESSION_ID = "payment_session_id"
REDIRECT_URL_MODE = "mode"
WEBHOOK_TYPE_PAYMENT = "PAYMENT_SUCCESS_WEBHOOK"
PAYMENT_STATUS_SUCCESS = "SUCCESS"
DATE_FORMAT = "SHORT_DATETIME_FORMAT"

SUPPORTED_CURRENCIES = ["INR"]
SUPPORTED_COUNTRY_CODES = [91]
