# Generated by Django 4.2.24 on 2025-09-13 04:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("pretix_cashfree", "0002_delete_paymentwebhookevent"),
    ]

    operations = [
        migrations.RenameField(
            model_name="paymentattempt",
            old_name="order_id",
            new_name="reference",
        ),
    ]
