{% load i18n %}
{% if refund_info %}
    <dl class='dl-horizontal'>
        <dt>{% trans "Request ID" %}</dt>
        <dd>
            <span id="{{ refund_info.refund_id }}_request_id" class="text-muted">{{ refund_info.x_request_id }}</span>  
            <button type="button" class="btn btn-default btn-xs btn-clipboard js-only" data-clipboard-target="#{{ refund_info.refund_id }}_request_id" data-original-title="" title="">
                <i class="fa fa-clipboard" aria-hidden="true"></i>
                <span class="sr-only">{% trans "Copy to clipboard" %}</span>
            </button>      
        </dd>

        <dt>{% trans "Refund ID" %}</dt>
        <dd>
            <span id="{{ refund_info.refund_id }}_refund_id" class="text-muted">{{ refund_info.refund_id }}</span>  
            <button type="button" class="btn btn-default btn-xs btn-clipboard js-only" data-clipboard-target="#{{ refund_info.refund_id }}_refund_id" data-original-title="" title="">
                <i class="fa fa-clipboard" aria-hidden="true"></i>
                <span class="sr-only">{% trans "Copy to clipboard" %}</span>
            </button>   
        </dd>

        <dt>{% trans "Order ID" %}</dt>
        <dd>
            <span id="{{ refund_info.refund_id }}_order_id" class="text-muted">{{ refund_info.order_id }}</span>  
            <button type="button" class="btn btn-default btn-xs btn-clipboard js-only" data-clipboard-target="#{{ refund_info.refund_id }}_order_id" data-original-title="" title="">
                <i class="fa fa-clipboard" aria-hidden="true"></i>
                <span class="sr-only">{% trans "Copy to clipboard" %}</span>
            </button>   
        </dd>

        <dt>{% trans "Updated At" %}</dt>
        <dd>{{ refund_info.updated_at }}</dd>
        
        <dt>{% trans "Status" %}</dt>
        <dd>
            {% if refund_info.refund_status == "ACTIVE" %}
                <span class="label label-info">{{ refund_info.refund_status|lower }}</span>
            {% elif refund_info.refund_status == "PAID" %}
                <span class="label label-success">{{ refund_info.refund_status|lower }}</span>
            {% elif refund_info.refund_status == "TERMINATED" %}
                <span class="label label-danger">{{ refund_info.refund_status|lower }}</span>
            {% elif refund_info.refund_status == "EXPIRED" %}
                <span class="label label-default">{{ refund_info.refund_status|lower }}</span>
            {% elif refund_info.refund_status == "TERMINATION_REQUESTED" %}
                <span class="label label-warning">{{ refund_info.refund_status|lower }}</span>
            {% else %}
                <span class="label label-default">{{ refund_info.refund_status|lower }}</span>
            {% endif %}
        </dd>

        
    </dl>
{% endif %}